<?php

/**
 * 
 *
 */

class Form_PageBasement extends Form_GuideFormBase
{

	public $lang;

	public $pageType;

	private $siteTree;
	
	private $siblings;


	public function __construct($parentId = null, $lang = null, $currentId = null, IComponentContainer $parent = null, $name = null)
	{
		parent::__construct($parent, $name);
		$this->siteTree = new SiteTree();
	}


	private function getFormatedSiblings($parentId = null, $lang = null, $currentId = null)
	{
		if ($parentId === 'null') {
			$parentId = null;
		} elseif ($parentId !== null) {
			$parentId = (int) $parentId;
		}

		$siblings = $this->siteTree->getSiblings($parentId, $lang, $currentId);

		$output = array('null' => '---- zařadit na začátek ----');
		foreach ($siblings as $ord => $sibling) {
			$output[$ord] = $sibling;
		}

		return $output;
	}


	private function initBasicFields()
	{
		$this->addText('name', 'Název stránky:', 30, 100)
			->addRule(Form::FILLED, 'Název stránky musí být vyplněn.');

		$this->addText('urlSuffix', 'Název v URL:', 30, 255)
			->addRule(Form::FILLED, 'Název v URL musí být vyplněn.')
			->addRule(Form::REGEXP, 'URL adresa není ve správném tvaru. Použít lze pouze znaky a-z, A-Z, 0-9, pomlčku a potržítko.','/^[a-z0-9_-]*$/i');


		$flatSiteTree = $this->siteTree->getFlatSiteTree($this->lang);

		$adjustedSiteTree = array();
		$adjustedSiteTree['null'] = '---- žádná ----';

		foreach ($flatSiteTree as $val) {
			$adjustedSiteTree[$val['pageId']] = str_repeat('--- ', $val['level']-1).$val['name'];
		}

		$this->addSelect('parent', 'Nadřazená stránka:', $adjustedSiteTree)
			->getControlPrototype()->onchange = "$(this).parents('form').find('input[name=\"loadSiblings\"]').ajaxSubmit()";

		$this->addSelect('placeBehind', 'Zařadit za stránku:', $this->siblings);

		$this->addSubmit('loadSiblings', 'Potvrdit')
				->setValidationScope(false)
				->getControlPrototype()->class = 'jsHidden';

		// SEO
		$this->addText('seoTitle', 'SEO titulek:', 30, 255);
		$this->addText('seoDescription', 'popis (description):', 30, 255);
		$this->addText('seoKeywords', 'klíčová slova (keywords):', 30, 255);
	}


	private function initExtendedFields()
	{
		$this->addCheckbox('published', 'Označit stránku za publikovanou')
			->setDefaultValue(true);

		$this->addCheckbox('shownInMenu', 'Zobrazovat stránku v menu')
			->setDefaultValue(true);

		$this->addCheckbox('homepage', Html::el('span')->setHtml('Nastavit jako novou úvodní stránku (pro jazykovou verzi <strong>'.$this->lang.'</strong>)'));
	}


	public function initCreateFormPart1()
	{
		$this->checkInitialization();

		$this->siblings = $this->getFormatedSiblings(null, $this->lang);

		$this->initBasicFields();

		$this->addGuideSubmit();

		$this->onSubmit[] = array($this, 'createFormPart1Submitted');

		$this->initialized = true;
	}


	public function initCreateFormPart2()
	{
		$this->checkInitialization();

		$this->initExtendedFields();

		$this->addGuideSubmit();

		$this->onSubmit[] = array($this, 'createFormPart2Submitted');

		$this->initialized = true;
	}
	


	public function createFormPart1Submitted(AppForm $form)
	{

		$parent = $form['parent']->getValue();
		if ($parent !== 'null') {
			$this->siblings = $this->getFormatedSiblings($form['parent']->getValue());
		} else {
			$this->siblings = $this->getFormatedSiblings(null, $this->lang);
		}
		$form['placeBehind']->setItems($this->siblings);

		if ($form['loadSiblings']->isSubmittedBy()) {

			$parent = $form['parent']->getValue();
			$parent = $parent === 'null' ? null : (int) $parent;

			$form['placeBehind']->setValue('null');

			if ($this->presenter->isAjax()) {
				$this->presenter->invalidateControl('pageBasementFormPart1');
			}
		} else {

			// zakládání stránky
			$pageBasement = new PageBasement();

			$values = $form->getValues();

			if ($values['parent'] !== 'null') {
				$parentPage = $pageBasement->getPageBasement($values['parent']);
				$values['url'] = $parentPage['url'].'/'.$values['urlSuffix'];
			} else {
				$values['url'] = $values['urlSuffix'];
				$parent = null;
			}
			unset($values['urlSuffix']);
			
			
			foreach (array('seoTitle', 'seoDescription', 'seoKeywords', 'parent') as $null) {
				if ($values[$null] === '' or $values[$null] === 'null') {
					unset($values[$null]);
				}
			}

			$values['page-type_type'] = $this->pageType;
			$values['lang'] = $this->lang;
			$values['homepage'] = 0;
			$values['published'] = 0;
			$values['shownInMenu'] = 0;
			$values['lastModified'] = dibi::datetime();
			$values['currentStep'] = 1;


			try {
				// inicializace persistentního parametru důležitého pro další kroky průvodce
				$this->presenter->pageId = $pageBasement->createPage($values);

				// naopak typ stránky již nebude zapotřebí (již je v databázi)
				$this->presenter->type = null;

				$this->goToNextStep(array('step' => (int) $this->currentStep + 1));

			} catch (DibiDriverException $e) {
				if ($e->getCode() === DibiMySqlDriver::ERROR_DUPLICATE_ENTRY) {
					$this->addError('Požadovaná URL adresa je již obsazena, zvolte prosím jinou.');
				} else {
					$this->addError($e->getMessage());
				}
			}
		}
	}

	public function createFormPart2Submitted(AppForm $form)
	{
		$pageBasement = new PageBasement();

		$values = $form->getValues();

		$pageBasement->setVisibility($this->pageId, $values['published'], $values['shownInMenu']);

		if ($values['homepage'] === true) {
			$siteExplorer = new SiteExplorer();
			$siteExplorer->setHomepage($this->pageId);
		}

		$this->goToNextStep(array('step' => (int) $this->currentStep + 1));		
	}


	public function initUpdateForm($parentId)
	{
		$this->checkInitialization();

		$this->siblings = $this->getFormatedSiblings($parentId, $this->lang, $this->pageId);

		$this->initBasicFields();

		$this->initExtendedFields();

		$this->addSubmit('send', 'Uložit');

		$this->onSubmit[] = array($this, 'updateFormSubmitted');

		$this->initialized = true;

		return $this;
	}


	public function updateFormSubmitted(AppForm $form)
	{

		$this->siblings = $this->getFormatedSiblings($form['parent']->getValue(), $this->lang, $this->presenter->pageId);
		$form['placeBehind']->setItems($this->siblings);

		if ($form['loadSiblings']->isSubmittedBy()) {

			$parent = $form['parent']->getValue();
			$parent = $parent === 'null' ? null : (int) $parent;

			$form['placeBehind']->setValue('null');

			if ($this->presenter->isAjax()) {
				$this->presenter->invalidateControl('updatePageBasementForm');
			}
		} else {

			$pageBasement = new PageBasement();

			$values = $form->getValues();

			foreach (array('seoTitle', 'seoDescription', 'seoKeywords', 'parent') as $null) {
				if ($values[$null] === '' or $values[$null] === 'null') {
					$values[$null] = null;
				}
			}


			if (isset($values['homepage'])) {
				if ($values['homepage'] === true) {
					$siteExplorer = new SiteExplorer();
					$siteExplorer->setHomepage($this->presenter->pageId);
				}
				unset($values['homepage']);
			}

			//var_dump($values);die();

			try {
				$pageBasement->updatePage($this->presenter->pageId, $values);
				$this->presenter->pageSuccessfullyUpdated();

			} catch (InvalidArgumentException $e) {
				$this->addError($e->getMessage());
			} catch (DibiDriverException $e) {
				if ($e->getCode() === DibiMySqlDriver::ERROR_DUPLICATE_ENTRY) {
					$this->addError('Požadovanou úpravu nelze provést, protože by způsobila kolizi v URL adresách.');
				} else {
					$this->addError($e->getMessage());
				}
			}

		}
		
	}

}