<?php

/**
 * 
 *
 */

class Admin_Pages_BasePresenter extends Admin_BasePresenter
{

	/** @persistent */
	public $type;

	/** @persistent */
	public $pageId;

	public $stepsCount = 4;

	protected $currentStep;

	protected $pageBasement;

	protected $pageBasementData;


	protected $scriptsByView = array(
		'createStep1' => array(
			'js/jquery-1.4.2.min.js',
			'js/jquery.nette.js',
			'js/jquery.ajaxform.js',
			'js/admin.init.js',
		),
		'createStep2' => array(
			'fckeditor/fckeditor.js',
		),
		'updatePageBasement' => array(
			'js/jquery-1.4.2.min.js',
			'js/jquery.nette.js',
			'js/jquery.ajaxform.js',
			'js/admin.init.js',
		),
		'updatePrivateContents' => array(
			'fckeditor/fckeditor.js',
		)
	);


	protected $tabs = array(
		'pageBasement' => 'Vlastnosti stránky',
		'privateContents' => 'Hlavní obsahy',
		'sharedContents' => 'Sdílení obsahů'
	);

	protected $currentTab;
	

	public static function getPersistentParams()
	{
		return array('pageId', 'type');
	}
	

	public function startup()
	{
		parent::startup();
		$this->langSwitcherDisabled(true);

		$this->pageBasement = new PageBasement();
	}


	public function actionCreate($step = null)
	{

		if ($step === null) {
			$this->currentStep = 1;
		} else {
			$this->currentStep = (int) $step;
		}

		// validace
		$pageId = $this->pageId;
		if ($this->currentStep > 1) {

			if ($pageId === null) {
				throw new ApplicationException('S požadovaným krokem průvodce vytvoření stránky není možné pracovat bez znalosti ID stránky.');
			}
			$this->pageBasementData = $this->pageBasement->getPageBasement($pageId);

			if ((int) $this->pageBasementData['currentStep'] !== $this->currentStep - 1) {
				throw new ApplicationException('S požadovaným krokem průvodce vytvoření stránky není možné u dané stránky pracovat.');
			}
		}

		
		$this->setView('createStep'.$this->currentStep);
		try {
			$this->template->setFile(dirname(__FILE__).'/../templates/Default/'.$this->view.'.phtml');
		} catch (FileNotFoundException $e) {}


		$pageTypes = new PageTypes();
		if ($this->type === null and $pageId !== null) {
			if ($this->pageBasementData !== null) {
				$this->pageBasementData = $this->pageBasement->getPageBasement($pageId);
			}
			$this->template->typeName = $pageTypes->getPageTypeName($this->pageBasementData['page-type_type']);
		} else {
			$this->template->typeName = $pageTypes->getPageTypeName($this->type);
		}

	}


	public function actionUpdate($pageId, $tab = null)
	{
		if ($pageId === null) {
			throw new ApplicationException('Nebylo specifikováno ID stránky, která se má upravovat.');
		}

		if ($tab === null) {
			$this->currentTab = 'pageBasement';
		} else {
			$this->currentTab = $tab;
		}

		$this->setView('update'.ucfirst($this->currentTab));
		try {
			$this->template->setFile(dirname(__FILE__).'/../templates/Default/'.$this->view.'.phtml');
		} catch (FileNotFoundException $e) {}

		$this->pageBasementData = $this->pageBasement->getPageBasement($pageId);
		$this->template->pageName = $this->pageBasementData['name'];

		$this->template->tabs = $this->tabs;
		$this->template->currentTab = $this->currentTab;
		
	}


	public function actionDelete($pageId)
	{
		$this->pageBasementData = $this->pageBasement->getPageBasement($pageId);
		try {
			$this->template->setFile(dirname(__FILE__).'/../templates/Default/delete.phtml');
		} catch (FileNotFoundException $e) {}
		$this->template->pageName = $this->pageBasementData['name'];
	}


	public function beforeRender()
	{
		parent::beforeRender();
		if (isset($this->scriptsByView[$this->view])) {
			$this->scripts = $this->scriptsByView[$this->view];
		}
	}


	protected function createComponentCreatePageBasementFormPart1()
	{
		if ($this->currentStep !== 1) {
			throw new ApplicationException('Definice základních vlastností stránky musí být prvním krokem průvodce tvorbou nové stránky.');
		}

		$form = new Form_PageBasement(null, $this->lang);

		$form->lang = $this->lang;
		$form->pageType = $this->type;
		$form->stepsCount = $this->stepsCount;
		$form->currentStep = 1;

		$form->initCreateFormPart1();

		return $form;
	}


	protected function createComponentCreatePageBasementFormPart2()
	{
		$form = new Form_PageBasement();

		$form->lang = $this->lang;
		$form->stepsCount = $this->stepsCount;
		$form->currentStep = $this->currentStep;

		$form->pageId = $this->pageId;

		$form->initCreateFormPart2();

		return $form;
	}


	protected function createComponentUpdatePageBasementForm()
	{
		if ($this->pageBasementData === null) {
			$this->pageBasementData = $this->pageBasement->getPageBasement($this->pageId);
		}

		$form = new Form_PageBasement();

		$form->lang = $this->lang;
		$form->pageId = $this->pageId;

		$form->initUpdateForm($this->pageBasementData['parent']);

		// nastavení aktuálních hodnot
		$urlParts = explode('/',$this->pageBasementData['url']);

		$this->pageBasementData['urlSuffix'] = $urlParts[count($urlParts)-1];

		$siteTree = new SiteTree();
		$this->pageBasementData['placeBehind'] = $siteTree->getForegoingSibling($this->pageBasementData['id']);

		if ((int) $this->pageBasementData['homepage'] === 1) {
			unset($form['homepage']);
		}

		$form->setDefaults($this->pageBasementData);

		return $form;

	}


	protected function createComponentCreatePrivateContentsForm()
	{
		$form = new Form_PrivateContents();

		$form->stepsCount = $this->stepsCount;
		$form->currentStep = $this->currentStep;

		$form->pageId = $this->pageId;

		$form->initCreateForm();

		return $form;

	}


	protected function createComponentUpdatePrivateContentsForm()
	{
		$form = new Form_PrivateContents();

		$form->pageId = $this->pageId;

		$form->initUpdateForm(true);

		return $form;		
	}


	protected function createComponentCreateSharedContentsForm()
	{

		$form = new Form_sharedContents();

		$form->stepsCount = $this->stepsCount;
		$form->currentStep = $this->currentStep;

		$form->pageId = $this->pageId;

		$form->initCreateForm();

		return $form;
		
	}


	protected function createComponentUpdateSharedContentsForm()
	{

		$form = new Form_sharedContents();

		$form->pageId = $this->pageId;

		$form->initUpdateForm();

		// nastavení aktuálních hodnot
		$sharedContents = new SharedContents();
		$shares = $sharedContents->getSharedContentsByPage($this->pageId);

		foreach ($shares as $share) {
			$form[Form_SharedContents::CONTENT_FORM_PREFIX.$share['id']]->setDefaultValue(true);
		}

		return $form;

	}


	public function createComponentDeleteForm()
	{
		$form = new Form_Delete();
		$form->initDeleteForm();
		return $form;
	}


	public function cancelRedirect()
	{
		$this->redirect(':Admin:default:');
	}


	public function deletePage()
	{
		try {
			$this->pageBasement->deletePage($this->pageId);

			$this->flashMessage('Stránka byla úspěšně odstraněna.');
			$this->redirect(':Admin:default:');
		} catch (InvalidArgumentException $e) {
			$this['deleteForm']->addError($e->getMessage());
		}
	}


	public function pageSuccessfullyCreated()
	{
		$this->flashMessage('Stránka byla úspěšně vytvořena');

		$this->pageBasement->updateCurrentStep($this->pageId, null);
		$this->redirect(':Admin:default:');
	}


	public function pageSuccessfullyUpdated()
	{
		$this->flashMessage('Změny byly úspěšně uloženy');
		$this->redirect('this');
	}

    
}