<?php

/**
 * 
 *
 */

abstract class Admin_BasePresenter extends BasePresenter
{

	/** @persistent */
	public $lang;

	protected $scripts = array();

	protected $siteSettings;


	public static function getPersistentParams()
	{
		return array('lang');
	}

	public function startup()
	{
		parent::startup();

		$user = Environment::getUser();
		$user->setNamespace('AdminModule');

		if (!$user->isAuthenticated()) {
			if ($user->getSignOutReason() === User::INACTIVITY) {
				$this->flashMessage('Byl(a) jste odhlášen(a) z důvodu déle trvající neaktivity. Přihlaste se prosím znovu.');
				$user->signOut(true);
			}
			$this->redirect(':Admin:Auth:');
		}

		$this->template->adminAuthenticated = true;

		$this->siteSettings = Environment::getConfig('siteSettings');

		//inicializace jazyka
		if ($this->lang === null) {
			$langs = explode(',', $this->siteSettings->langs);
			$this->lang = $langs[0];
		}

		$this->langSwitcherDisabled(false);

	}


	public function beforeRender()
	{
		parent::beforeRender();

		$this->template->siteName = $this->siteSettings->siteName;
		$this->template->lang = $this->lang;
		$this->template->langs = explode(',', $this->siteSettings->langs);

		if (String::startsWith($this->getName(), 'Admin:Pages')) {
			$this->template->currentPresenter = 'Admin:Default';
		} else {
			$this->template->currentPresenter = $this->getName();
		}		
	}


	public function afterRender()
	{
		$this->template->scripts = $this->scripts;
	}


	protected function langSwitcherDisabled($bool)
	{
		$this->template->langSwitcherDisabled = (boolean) $bool;
	}


	protected function langSwitcherHidden($bool)
	{
		$this->template->langSwitcherHidden = (boolean) $bool;
	}


	public function handleLogout()
	{
		$user = Environment::getUser();
		$user->setNamespace('AdminModule');

		$user->logout();
		$this->flashMessage('Byl(a) jste bezpečně odhlášen(a).');

		$this->redirect(':Admin:Auth:');
	}

}