<?php

/**
 * 
 *
 */

class Admin_AluPricesPresenter extends Admin_BasePresenter
{
	const YEAR_RANGE_FROM = 2010;

	const YEAR_RANGE_TO = 2030;


	public function startup()
	{
		parent::startup();
		$this->langSwitcherHidden(true);
	}


	public function handleGenerateGraph()
	{
		$aluPrice = new AluPrice(AluPrice::ERRMODE_EXCEPTIONS);
		$aluPrice->generateGraph();
		$this->flashMessage('Graf vývoje cen byl úspěšně aktualizován');
		$this->redirect('this');
	}


	public function actionDefault($year)
	{
		// validace roku v URL
		if (!in_array($year, range(self::YEAR_RANGE_FROM, self::YEAR_RANGE_TO))) {
			$this->redirect('this', self::YEAR_RANGE_FROM);
		}
	}


	public function renderDefault($year)
	{
		$this->template->year = $year;
	}

	protected function createComponentSelectYearForm()
	{
		$form = new AppForm();

		$form->setMethod('get');

		$years = array();
		for ($i = self::YEAR_RANGE_FROM; $i <= self::YEAR_RANGE_TO; $i++) {
			$years[$i] = $i;
		}
		$form->addSelect('year', null, $years)->getControlPrototype()->class = 'big';

		$year = $this->getParam('year');
		if ($year !== null) {
			$form['year']->setDefaultValue($year);
		}
		$form->addSubmit('send', 'Zobrazit rok');

		return $form;
	}


	protected function createComponentValuesForm()
	{
		$form = new AppForm();
		
		$months = array(1 => 'Leden', 2 => 'Únor', 3 => 'Březen', 4 => 'Duben', 5 => 'Květen', 6 => 'Červen', 7 => 'Červenec', 8 => 'Srpen', 9 => 'Září', 10 => 'Říjen', 11 => 'Listopad', 12 => 'Prosinec');
		foreach ($months as $index => $name) {
			$form->addText('month_'.$index, $name.':', 10, 9)
				->addCondition(Form::FILLED)
				->addRule(Form::FLOAT, 'Hodnota u měsíce '.$name.' není platné desetinné číslo.');
			$form['month_'.$index]->getControlPrototype()->class = 'text-center';
		}
		$form->addSubmit('send', 'Uložit hodnoty');

		$year = $this->getParam('year');
		if ($year !== null) {
			$aluPrice = new AluPrice(AluPrice::ERRMODE_EXCEPTIONS);
			$values = $aluPrice->findYearPrices($year);
			foreach ($values as $value) {
				$form['month_'.$value['month']]->setDefaultValue($value['price']);
			}
		}
		$form->onSubmit[] = array($this, 'valuesFormSubmitted');

		return $form;
	}


	public function valuesFormSubmitted(AppForm $form)
	{

		$values = $form->getValues();
		$year = $this->getParam('year');

		$aluPrice = new AluPrice(AluPrice::ERRMODE_EXCEPTIONS);

		foreach ($values as $key => $val) {

			if ($val === '') {
				$aluPrice->deletePrice($year, substr($key, 6));
			} else {
				if (substr($key, 0, 6) === 'month_') {
					$aluPrice->insertPrice(array('year' => $year, 'month' => substr($key, 6), 'price' => $val));
				}
			}
		}

		$aluPrice->generateGraph();

		$this->flashMessage('Změny byly úspěšně uloženy, graf byl úspěšně aktualizován');
		$this->redirect('this');
	}

}