<?php

/**
 * 
 *
 */

class Admin_NewsPresenter extends Admin_BasePresenter
{

	public function renderDefault()
	{

		$news = new News();

		$paginator = $this['vp']->getPaginator();
		$paginator->itemsPerPage = 20;
		$paginator->itemCount = $news->getNewsCount($this->lang);

		$this->template->news = $news->getNewsOverview($this->lang, $paginator->offset, $paginator->itemsPerPage, false);

	}


	public function renderCreate()
	{
		$this->langSwitcherDisabled(true);
		$this->scripts = array(
			'fckeditor/fckeditor.js'
		);
		$this->template->formComponentName = 'createNewsForm';
	}


	public function renderUpdate($id)
	{
		$this->langSwitcherDisabled(true);
		$this->scripts = array(
			'fckeditor/fckeditor.js'
		);
		$this->template->formComponentName = 'updateNewsForm';

		$this->setView('create');

		$news = new News();
		$newsItem = $news->getNews($id);

		if ($newsItem['content'] !== null) {
			$newsItem['perex'] .= '<div style="page-break-after: always;"><span style="display: none;">&nbsp;</span></div>'.$newsItem['content'];
		}
		$this['updateNewsForm']->setDefaults($newsItem);

	}


	public function renderDelete($id)
	{
		$news = new News();
		$newsItem = $news->getNews($id);

		$this->template->title = $newsItem['title'];
	}


	private function createNewsFormBase()
	{
		$form = new AppForm();

		$form->addText('title', 'Titulek:', 50, 255)
			->addRule(Form::FILLED, 'Titulek musí být uveden');

		$form->addText('url', 'Název v URL:', 30, 255)
			->addRule(Form::FILLED, 'Název v URL musí být vyplněn.')
			->addRule(Form::REGEXP, 'URL adresa není ve správném tvaru. Použít lze pouze znaky a-z, A-Z, 0-9, pomlčku a potržítko.','/^[a-z0-9_-]*$/i');

		$form->addCheckbox('published', 'Označit novinku za publikovanou');

		$form->addTextArea('perex', 'Obsah:', 70, 12);

		return $form;		
	}


	protected function createComponentCreateNewsForm()
	{
		$form = $this->createNewsFormBase();
		$form->addSubmit('send', 'Publikovat');
		$form->onSubmit[] = array($this, 'createNewsFormSubmitted');

		return $form;
	}


	public function createNewsFormSubmitted(AppForm $form)
	{
		$values = $form->getValues();

		$breakPosition = strpos($values['perex'], '<div style="page-break-after: always;"><span style="display: none;">&nbsp;</span></div>');

		if ($breakPosition !== false) {
			$values['content'] = substr($values['perex'], $breakPosition + 87);
			$values['perex'] = substr($values['perex'], 0, $breakPosition);
		} else {
			$values['content'] = null;
		}

		$values['lang'] = $this->lang;
		$values['date'] = dibi::date();

		$news = new News();

		try {
			$news->createNews($values);
			$this->flashMessage('Novinka byla úspěšně publikována');
			$this->redirect('default');

		} catch (DibiDriverException $e) {
			if ($e->getCode() === DibiMySqlDriver::ERROR_DUPLICATE_ENTRY) {
				$form->addError('Požadovaná URL adresa je již obsazena, zvolte prosím jinou.');
			} else {
				$form->addError($e->getMessage());
			}
		};

	}


	protected function createComponentUpdateNewsForm()
	{
		$form = $this->createNewsFormBase();
		$form->addSubmit('send', 'Uložit');
		$form->onSubmit[] = array($this, 'updateNewsFormSubmitted');

		return $form;
	}


	public function updateNewsFormSubmitted(AppForm $form)
	{
		$values = $form->getValues();

		$breakPosition = strpos($values['perex'], '<div style="page-break-after: always;"><span style="display: none;">&nbsp;</span></div>');

		if ($breakPosition !== false) {
			$values['content'] = substr($values['perex'], $breakPosition + 87);
			$values['perex'] = substr($values['perex'], 0, $breakPosition);
		} else {
			$values['content'] = null;
		}

		$news = new News();

		try {

			$news->updateNews($this->getParam('id'), $values);
			$this->flashMessage('Novinka byla úspěšně upravena');
			$this->redirect('this');

		} catch (DibiDriverException $e) {
			if ($e->getCode() === DibiMySqlDriver::ERROR_DUPLICATE_ENTRY) {
				$form->addError('Požadovaná URL adresa je již obsazena, zvolte prosím jinou.');
			} else {
				$form->addError($e->getMessage());
			}
		};

	}


	protected function createComponentDeleteForm()
	{
		$form = new Appform();

		$form->addProtection();

		$form->addSubmit('ok', 'Ano');
		$form->addSubmit('no', 'Ne');

		$form->onSubmit[] = array($this, 'deleteFormSubmitted');

		return $form;
	}



	public function deleteFormSubmitted(AppForm $form)
	{
		if ($form['ok']->isSubmittedBy()) {
			$news = new News();
			$news->deleteNews($this->getParam('id'));

			$this->flashMessage('Novinka byla úspěšně odstraněna');
		}
		$this->redirect('default');

	}

}