<?php

/**
 * Základ pro stránky plně spravovatelné z administrace
 *
 */

abstract class ManagedPageBasePresenter extends BasePresenter implements ISiteStateHolder
{

	/** @persistent */
	public $lang;

	/** @persistent */
	public $pageId;

	// @var ISiteState
	private $siteState;


	public static function getPersistentParams()
	{
		return array('lang', 'pageId');
	}


	public function startup()
	{
		parent::startup();

		if ((int) $this->getParam('published') !== 1) {
			throw new BadRequestException('Požadovaná stránka nebyla nalezena.');
		}

		if ((int) $this->getParam('homepage') !== 1) {
			$this->autoCanonicalize = false;
		}

		$siteExplorer = Environment::getService('Core\ISiteExplorer');

		$this->siteState = new SiteState();

		$this->siteState['lang'] = $this->lang;
		$this->siteState['pageId'] = $this->pageId;
		$this->siteState['homepageInActiveLang'] =  $siteExplorer->getHomepage($this->lang);
		$this->siteState['activePageIsHomepage'] = (int) $this->siteState['homepageInActiveLang']['pageId'] === $this->pageId ? true : false;

		$this->template->activePageIsHomepage = $this->siteState['activePageIsHomepage'];
	}



	public function getTranslator()
	{
		$cache = Environment::getCache('CMF.Translator');

		if (isset($cache['Translator.'.$this->lang])) {
			$translator = $cache['Translator.'.$this->lang];
		} else {
			$translator = new GettextTranslator(APP_DIR.'/locale/locale.'.$this->lang.'.mo');
			$cache->save('Translator.'.$this->lang, $translator, array(
				Cache::TAGS => array('Translator.'.$this->lang)
			));
		}
		return $translator;
	}



	public function beforeRender()
	{
		$this->template->setTranslator($this->getTranslator());
		$this->template->lang = $this->lang;
	}


	/**
	 * @see ISiteState:getSiteState()
	 *
	 */
	public function getSiteState()
	{
		return $this->siteState;
	}


	protected function createComponent($name)
	{
		$component = parent::createComponent($name);

		if ($component instanceof ISiteStateClientModelsHolder) {
			$component->initModels($this->siteState);
		}
		return $component;
	}


	protected function createComponentHpNews()
	{
		return new HpNewsControl();
	}


	/************ Průmyslová zóna :) ************/

	protected function createComponentLangSwitcher()
	{
		return new LangSwitcherControl();
	}

	protected function createComponentPageBasement()
	{
		return new PageBasementControl();
	}

	protected function createComponentGlobalContents()
	{
		return new GlobalContentsControl();
	}

	protected function createComponentSharedContents()
	{
		return new SharedContentsControl();
	}

	protected function createComponentPrivateContents()
	{
		return new PrivateContentsControl();
	}

	protected function createComponentSiteTree()
	{
		return new SiteTreeControl();
	}


	protected function createComponentAluPrices()
	{
		return new AluPriceControl();
	}

}