<?php

/**
 * 
 *
 */

class Front_OrderPresenter extends ManagedPageBasePresenter
{

	public function renderDefault()
	{
		
	}


	protected function createComponentOrderForm()
	{
		$form = new AppForm();

		$form->addText('name', 'Jméno / Název firmy:', 50)
			->addRule(Form::FILLED, 'Musíte zadat jméno nebo název firmy.');

		$form->addText('street', 'Ulice:', 50)
			->addRule(Form::FILLED, 'Ulice musí být vyplněna.');

		$form->addText('city', 'Město:', 50)
			->addRule(Form::FILLED, 'Město musí být vyplněno.');

		$form->addText('zip', 'PSČ:', 10)
			->addRule(Form::FILLED, 'PSČ musí být vyplněno.');

		$form->addText('ico', 'IČO*:', 50);
		$form->addText('dic', 'DIČ*:', 50);

		$form->addText('contactPerson', 'Kontaktní osoba:', 50);

		$form->addText('phone', 'Telefon:', 50)
			->addRule(Form::FILLED, 'Telefon musí být vyplněn.');

		$form->addText('fax', 'Fax:', 50);

		$form->addText('email', 'E-mail:', 50)
			->addCondition(Form::FILLED)
			->addRule(Form::EMAIL, 'Uvedená e-mailová adresa není platná.');


		// zboží
		for ($i = 1; $i <= 7; $i++) {
			$form->addText('item'.$i.'_stuff', 'materiál', 18);
			if ($i === 1) {
				$form['item'.$i.'_stuff']->addRule(Form::FILLED, 'Objednávka musí obsahovat alespoň jednu položku (vyplněný materiál a tloušťku/průměr).');
			}
			$form->addText('item'.$i.'_size', 'tloušťka/průměr', 14);
			if ($i === 1) {
				$form['item'.$i.'_size']->addRule(Form::FILLED, 'Objednávka musí obsahovat alespoň jednu položku (vyplněný materiál a tloušťku/průměr).');
			}
			$form->addText('item'.$i.'_width', 'šířka', 10);
			$form->addText('item'.$i.'_length', 'délka', 10);
			$form->addText('item'.$i.'_kg', 'kg*', 7);
			$form->addText('item'.$i.'_ks', 'ks*', 7);
		}


		$form->addText('arrivalDate', 'Požadovaný termín:', 50);
		$form->addText('transport', 'Způsob dopravy:', 50);
		$form->addText('date', 'Datum:', 50)->setDefaultValue(date('d.m.Y'));
		$form->addText('indication', 'Vaše označení objednávky:', 50);

		$form->addTextArea('notes', 'Poznámky:', 50, 5)->getControlPrototype()->class = 'textarea';

		$form->addSubmit('send', 'Odeslat objednávku')->getControlPrototype()->class = 'submit';

		$form->onSubmit[] = array($this, 'orderFormSubmitted');

		return $form;
	}


	public function orderFormSubmitted(AppForm $form)
	{
		$values = $form->getValues();

		$body = file_get_contents(APP_DIR.'/order.html');

		$id = OrderCounter::get();
		$body = str_replace("%%% id %%%", $id, $body);

		foreach ($values as $field => $value) {
			$body = str_replace("%%% $field %%%", $value !== '' ? $value : '<span style="color:#ccc">—</span>', $body);
		}

		$itemsHtml = '';
		$subfields = array('stuff', 'size', 'width', 'length', 'kg', 'ks');
		for ($i = 1; $i <= 7; $i++) {
			if ($values['item'.$i.'_stuff'] !== '' or $values['item'.$i.'_size'] !== '' or $values['item'.$i.'_width'] !== '' or $values['item'.$i.'_length'] !== '' or $values['item'.$i.'_kg'] !== '' or $values['item'.$i.'_ks'] !== '') {
				$itemsHtml .= '<tr>';
				foreach ($subfields as $field) {
					$itemsHtml .= '<td>'.($values['item'.$i.'_'.$field] !== '' ? $values['item'.$i.'_'.$field] : '<span style="color:#ccc">—</span>').'</td>';
				}
				$itemsHtml .= '</tr>';
			}
		}
		$body = str_replace("%%% items %%%", $itemsHtml, $body);


		$mail = new Mail;
		$mail->setFrom('Objednávky BP <noreply@vse.cz>');

		$settings = Environment::getService('Core\Settings');

		$mail->addTo($settings->email);
		if ($settings->email_cc !== null) {
			$mail->addCc($settings->email_cc);
		}
		$mail->setSubject('Objednávka z webové prezentace Alcom Alval (ID '.$id.')');
		$mail->setHtmlBody($body);

		try {
			$mail->send();
			$this->flashMessage('Vaše objednávka byla úspěšně přijata, děkujeme.');
			OrderCounter::advance();
			$this->redirect('this');
		} catch (Exception $e) {
			if (get_class($e) === 'AbortException') {
				throw $e;
			} else {
				$form->addError('Omlouváme se, ale při odesílání Vaší objednávky došlo k chybě.');
			}
		}
	}

}