<?php

/**
 * 
 *
 */

class SiteTreeControl extends Control implements ISiteStateClientModelsHolder
{

	private $siteTree;

	private $langTemplateVarsInitialized = false;

	private $crumbsAppendings = array();

	public $activeItemAsLink = false;

	//private $defaultLangInUrl;

	//private $langs;

	public function __construct(IComponentContainer $parent = null, $name = null)
	{
		parent::__construct($parent, $name);

		// inicializace modelu není lazy, protože lazy je už vlastní vytváření instance komponenty
		$this->siteTree = new SiteTree();
	}


	public function initModels(ISiteState $siteState)
	{
		$this->siteTree->initSiteState($siteState);
	}


	private function initLangTemplateVars()
	{
		if (!$this->langTemplateVarsInitialized) {

			$langs = explode(',',Environment::getConfig('siteSettings')->langs);
			$siteStateLang = $this->siteTree->siteState['lang'];

			$this->template->operatingWithNonDefaultLang = $siteStateLang !== $langs[0] ? true : false;

			$this->template->langPrefix = '';

			if (Environment::getConfig('siteSettings')->defaultLangInUrl) {
				$this->template->langPrefix = $this->siteTree->siteState['lang'].'/';
			} else {
				if ($this->template->operatingWithNonDefaultLang) {
					$this->template->langPrefix = $siteStateLang.'/';
				}
			}

			$this->langTemplateVarsInitialized = true;
		}
	}


	public function renderCrumbs()
	{

		$this->initLangTemplateVars();

		$ancestors = $this->siteTree->getAncestors(true);

		$oldestAncestor = reset($ancestors);

		if (!$this->siteTree->siteState['activePageIsHomepage'] and $oldestAncestor['pageId'] !== $this->siteTree->siteState['homepageInActiveLang']['pageId']) {
			$homepage = $this->siteTree->siteState['homepageInActiveLang'];
			$homepage['homepage'] = 1; // pro podmínky v šabloně
			array_unshift($ancestors, $homepage);
		}

		if (!empty($this->crumbsAppendings)) {
			foreach($this->crumbsAppendings as $appending) {
				$ancestors[] = $appending;
			}
		}

		$this->template->crumbs = $ancestors;
		$this->template->setFile(dirname(__FILE__).'/crumbs.phtml');
		$this->template->render();

	}


	public function renderMenu ($parentOrdInAncestors = null, $depth = null)
	{

		$this->initLangTemplateVars();

		if ($parentOrdInAncestors !== null) {

			$ancestors = $this->siteTree->getAncestors(true);
			reset($ancestors);
			$parent = false;
			for ($i = 1; $i <= $parentOrdInAncestors; $i++) {
				$parent = current($ancestors);
				next($ancestors);
			}
			if ($parent !== false) {
				$parentId = $parent['pageId'];
			} else {
				return;
			}
			$this->template->globalNav = false;

		} else {
			$parentId = null;
			$this->template->globalNav = true;
		}
		
		$this->template->menu = $this->siteTree->getSiteSubtree($parentId, $depth, true);

		$this->template->setFile(dirname(__FILE__).'/menu.phtml');
		$this->template->render();
	}


	public function appendCrumbItem($item)
	{
		if (!isset($item['url']) or !isset($item['name']) or !isset($item['homepage'])) {
			throw new InvalidArgumentException('Navigační stopa musí obsahovat minimálně název, URL a příznak, zda se jedná či nejená o domovskou stránku.');
		}
		$this->crumbsAppendings[] = $item;
	}

}