<?php

/**
 * Model pro práci se základními atributy stránky (její název, jazyk, description, titulek...)
 *
 */


class PageBasement extends SiteStateClientBaseModel
{

	/**
	 * Vrací pole všech atributů stránky o požadovaném ID
	 *
	 */
	public function getPageBasement($pageId = null)
	{
		if ($pageId === null) {
			$pageId = $this->siteState['pageId'];
		}
		if (($page = dibi::fetch('SELECT * FROM [page] WHERE [id] = %i', $pageId)) === false) {
			throw new InvalidArgumentException('Požadovaná stránka nebyla nalezena.');
		}
		return (array) $page;
	}


	/**
	 *
	 *
	 */
	public function createPage($values)
	{
		$siteTree = new SiteTree();

		$query = dibi::update('page', array('ord%sql' => '[ord] + 1'));
		if (isset($values['parent'])) {
			$query->where('[parent] = %i', $values['parent']);
		} else {
			$query->where('[parent] IS NULL');
		}
		if ($values['placeBehind'] !== 'null') {
			$values['placeBehind']++;
			// vkládáme stránku mezi stávající, a proto posuneme ty, které mají být až za ní
			$query->and('[ord] >= %i', $values['placeBehind']);
		} else {
			$values['placeBehind'] = 0;
		}
		$query->execute();

		$values['ord'] = $values['placeBehind'];
		unset($values['placeBehind']);

		dibi::query('INSERT INTO [page]', $values);
		$pageId = dibi::getInsertId();

		$siteTree->normalizeOrders(isset($values['parent']) ? $values['parent'] : null);

		$siteTree->rebuildTraversalTrees($values['lang']);

		// zakládání privátních obsahů
		$privateContents = new PrivateContents();
		$privateContents->createPrivateContents($pageId, $values['page-type_type']);

		return $pageId;
	}


	public function updatePage($pageId, $values)
	{

		dibi::begin();

		if ($values['parent'] == $pageId) {
			throw new InvalidArgumentException('Stránka nemůže být sama sobě nadřazenou stránkou.');
		}

		$pages = dibi::query('
			SELECT [id], [lang], [url], [ord], [family], [left], [right], [parent]
			FROM [page]
			WHERE [id] IN (%i', $pageId, ', %i', $values['parent'], ')'
		)->fetchAssoc('id');

		if ($values['parent'] !== null and $pages[$values['parent']]['family'] == $pages[$pageId]['family'] and ($pages[$values['parent']]['left'] > $pages[$pageId]['left'] and $pages[$values['parent']]['right'] < $pages[$pageId]['right'])) {
			throw new InvalidArgumentException('Není povoleno nastavit jako nadřazenou stránku některou z podřazených. K dosažení požadované struktury použijte prosím jiný postup.');
		}

		// příprava místa (ord) pro správné zařazení přesouvané stránky
		$query = dibi::update('page', array('ord%sql' => '[ord] + 1'));
		if ($values['parent'] !== null) {
			$query->where('[parent] = %i', $values['parent']);
		} else {
			$query->where('[parent] IS NULL');
		}
		if ($values['placeBehind'] !== 'null') {
			$values['placeBehind']++;
			// vkládáme stránku mezi stávající, a proto posuneme ty, které mají být až za ní
			$query->and('[ord] >= %i', $values['placeBehind']);
		} else {
			$values['placeBehind'] = 0;
		}

		$query->execute();

		$values['ord'] = $values['placeBehind'];
		unset($values['placeBehind']);

		// nová URL adresa
		if ($values['parent'] !== null) {
			$values['url'] = dibi::fetchSingle('SELECT [url] FROM [page] WHERE [id] = %i', $values['parent']).'/'.$values['urlSuffix'];
		} else {
			$values['url'] = $values['urlSuffix'];
			$parent = null;
		}
		unset($values['urlSuffix']);


		try {
			dibi::query('UPDATE [page] SET ', $values, 'WHERE [id] = %i', $pageId);
		} catch (DibiDriverException $e) {
			dibi::rollback();
			throw $e;
		}


		$siteTree = new SiteTree();

		$siteTree->normalizeOrders(isset($values['parent']) ? $values['parent'] : null);
		$siteTree->normalizeOrders($pages[$pageId]['parent']);

		$siteTree->rebuildTraversalTrees($pages[$pageId]['lang']);

		// úprava URL adres dětí

		$rebuildedPage = dibi::fetch('SELECT [left], [right], [family] FROM [page] WHERE [id] = %i', $pageId);

		$children = dibi::fetchAll('
			SELECT [id], [url]
			FROM [page]
			WHERE [left] BETWEEN %i', $rebuildedPage['left'], ' + 1 AND %i', $rebuildedPage['right'], ' - 1', 'AND [family] = %i', $rebuildedPage['family']
		);
		foreach ($children as $child) {
			$child['url'] = str_replace($pages[$pageId]['url'], $values['url'], $child['url'], $matched);
			try {
				dibi::query('UPDATE [page] SET [url] = %s', $child['url'], 'WHERE [id] = %i', $child['id']);
			} catch (DibiDriverException $e) {
				dibi::rollback();
				throw $e;
			}
		}

		// není zapotřebí znovu přepočítávat stromy, stačí jen pro jistotu ještě jednou vymazat cache
		$cache = Environment::getCache('CMF.Core');
		$cache->clean(array(
			Cache::TAGS => array('SiteTree.'.$pages[$pageId]['lang'])
		));

		dibi::commit();

	}


	public function deletePage($pageId)
	{

		// kontrola, zda je možné stránku vůbec smazat
		if (dibi::fetchSingle('SELECT COUNT([id]) FROM [page] WHERE [parent] = %i', $pageId) != 0) {
			throw new InvalidArgumentException('Stránku nelze odstranit, protože obsahuje podřazené stránky. Ty je zapotřebí přesunout.');
		}

		$page = dibi::fetch('SELECT [parent],[lang] FROM [page] WHERE [id] = %i', $pageId);
		// díky kaskádovým cizím klíčům není zapotřebí mazat nic jiného
		dibi::query('DELETE FROM [page] WHERE [id] = %i', $pageId);
		
		$siteTree = new SiteTree();

		$siteTree->normalizeOrders($page['parent']);
		$siteTree->rebuildTraversalTrees($page['lang']);
		
	}


	public function updateCurrentStep($pageId, $currentStep)
	{
		if ($currentStep === null) {
			dibi::query('UPDATE [page] SET [currentStep] = null');
		} else {
			dibi::query('UPDATE [page] SET [currentStep] = %i', $currentStep, 'WHERE [id] = %i', $pageId);
		}
		$cache = Environment::getCache('CMF.Core');
		$cache->clean(array(
			Cache::TAGS => array('SiteTree.'.dibi::fetchSingle('SELECT [lang] FROM [page] WHERE [id] = %i', $pageId))
		));
	}


	public function setVisibility($pageId, $published, $shownInMenu)
	{
		if (($lang = dibi::fetchSingle('SELECT [lang] FROM [page] WHERE [id] = %i', $pageId)) !== false) {
			dibi::query('UPDATE [page] SET [published] = %b', $published, ', [shownInMenu] = %b', $shownInMenu, 'WHERE [id] = %i', $pageId);

			$siteTree = new SiteTree();
			$siteTree->rebuildTraversalTrees($lang);
		} else {
			throw new InvalidArgumentException('Požadovaná stránka nebyla nalezena');
		}
	}

}