<?php

/**
 *
 *
 */

class Settings
{

	private $settings;

	private $defaultSettingsProperty;

	private $cache;


	public function __construct()
	{
		$this->cache = Environment::getCache('CMF.Core');
		if (isset($this->cache['Settings'])) {
			$this->settings = $this->cache['Settings'];
		} else {
			if (!is_file(APP_DIR.'/settings.json')) {
				if (!copy(APP_DIR.'/settings.default.json', APP_DIR.'/settings.json')) {
					throw new IOException('Došlo k chybě při vytváření souboru s nastavením.');
				}
			}
			$this->settings = $this->parseSettingsFile(APP_DIR.'/settings.json');

			$this->cache->save('Settings', $this->settings, array(
				Cache::TAGS => array('Settings')
			));
		}
	}


	private function getDefaultSettings()
	{
		if ($this->defaultSettingsProperty === null) {
			$this->defaultSettingsProperty = $this->parseSettingsFile(APP_DIR.'/settings.default.json');
		}
		return $this->defaultSettingsProperty;
	}


	private function parseSettingsFile($file)
	{
		$settings = file_get_contents($file);
		if ($settings === false) {
			throw new IOException('Došlo k chybě při otevírání souboru s nastavením.');
		}
		$settings = json_decode($settings, true);
		if ($settings === null) {
			throw new IOException('Došlo k chybě při parsování souboru s nastavením.');
		}
		return $settings;
	}


	public function getAllSettings()
	{
		return $this->settings;
	}


	public function  __get($name)
	{
		if (!array_key_exists($name, $this->settings)) {

			$defaultSettings = $this->getDefaultSettings();
			if (array_key_exists($name, $defaultSettings)) {
				$this->register($name, $defaultSettings[$name]);
			} else {
				throw new InvalidArgumentException('Požadované nastavení nebylo nalezeno.');
			}
		}
		return $this->settings[$name];
	}


	public function __set($name, $value)
	{
		if (array_key_exists($name, $this->getDefaultSettings())) {
			$this->settings[$name] = $value;
		} else {
			throw new InvalidArgumentException('Nelze nastavit hodnotu neexistujícího nastavení.');
		}
	}


	private function register($name, $value)
	{
		if (array_key_exists($name, $this->settings)) {
			throw new InvalidArgumentException('Registrované nastavení již existuje.');
		} else {
			$this->settings[$name] = $value;
		}
	}


	/*public function unregister($name)
	{
	}*/


	public function store()
	{
		if (file_put_contents(APP_DIR.'/settings.json', json_encode($this->settings)) === false) {
			throw new IOException('Došlo k chybě při ukládání nastavení.');
		}
		$this->cache->clean(array(
			Cache::TAGS => array('Settings')
		));
	}

}