<?php

/**
 * @see ISiteExplorer
 *
 */

class SiteExplorer extends Object implements ISiteExplorer
{

	/**
	 * @see ISiteExplorer:matchPageByUrl()
	 *
	 */
	public function matchPageByUrl($lang, $url)
	{
		if (($page = dibi::fetch('
			SELECT [id] [pageId], [homepage], [presenter], [action], [published]
			FROM [page] JOIN [page-type] ON [page.page-type_type] = [page-type.type]
			WHERE [lang] = %s', $lang, 'AND [url] = %s', $url
		)) === false) {
			throw new InvalidArgumentException('Požadovaná stránka nebyla nalezena.');
		}
		return (array) $page;
	}


	/**
	 * @see ISiteExplorer:matchPagesByUrlFragment()
	 *
	 */
	public function matchPagesByUrlFragment($lang, $urlFragment)
	{
		$pages = dibi::query('
			SELECT [id] [pageId], [homepage], [url], [presenter], [action], [published], [overloadAbility]
			FROM [page] JOIN [page-type] ON [page.page-type_type] = [page-type.type]
			WHERE [lang] = %s', $lang, 'AND (LOCATE([url], %s) = 1', $urlFragment, ' OR [homepage] = 1)AND [overloadAbility] IS NOT NULL
		')->fetchAssoc('url');
		if (empty($pages)) {
			throw new InvalidArgumentException('Nebyly nalezeny žádné stránky obsahující požadovaný URL fragment.');
		}
		return $pages;
	}


	/**
	 * @see ISiteExplorer:matchPageById()
	 *
	 */
	public function matchPageById($id)
	{
		if (($page = dibi::fetch('
			SELECT [lang], [url], [homepage], [presenter], [action], [overloadAbility], [published]
			FROM [page] JOIN [page-type] ON [page.page-type_type] = [page-type.type]
			WHERE [id] = %i', $id
		)) === false) {
			throw new InvalidArgumentException('Požadovaná stránka nebyla nalezena.');
		}
		return (array) $page;
	}


	/**
	 * @see ISiteExplorer:getSiteName()
	 *
	 */
	public function getSiteName()
	{
		return Environment::getConfig('siteSettings')->siteName;
	}

	
	public function getHomepage($lang)
	{
		return dibi::fetch("
			SELECT [id] [pageId], [homepage], [page.name], [lang], [url], [presenter], [action], [published]
			FROM [page] JOIN [page-type] ON [page.page-type_type] = [page-type.type]
			WHERE [homepage] = 1 AND [lang] = %s", $lang
		);
	}


	public function setHomepage($pageId)
	{
		if (($lang = dibi::query('SELECT [lang] FROM [page] WHERE [id] = %i', $pageId)->fetchSingle()) !== false) {
			dibi::query('UPDATE [page] SET [homepage] = 0 WHERE [lang] = %s', $lang);
			dibi::query('UPDATE [page] SET [homepage] = 1 WHERE [id] = %i', $pageId);
		} else {
			throw new InvalidArgumentException('Požadovaná stránka nebyla nalezena.');
		}
	}

	public function getFirstPageByType($lang, $type)
	{
		return dibi::fetchSingle('SELECT [url] FROM [page] WHERE [page-type_type] = %s', $type, 'AND [lang] = %s', $lang, 'AND published = 1 ORDER BY [id]');
	}

}