<?php

/**
 * Router, který umí pracovat se strukturou webu uloženou v databázi
 *
 */

class DatabaseRouter extends Object implements IRouter
{

	private $flags;
	private $defaultLangInUrl;
	private $siteExplorer;


	public function __construct($flags = 0)
	{
		$this->flags = $flags;
		$this->defaultLangInUrl = Environment::getConfig('siteSettings')->defaultLangInUrl;
		$this->siteExplorer = Environment::getService('Core\ISiteExplorer');
	}


	/**
	 * Převádí HTTP požadavek na instanci objektu PresenterRequest
	 *
	 */
	public function match(IHttpRequest $httpRequest)
	{
		// podle URL budeme hledat požadovanou stránku v databázi
		$relativeUri = $httpRequest->getUri()->getRelativeUri();

		$langs = explode(',', Environment::getConfig('siteSettings')->langs);

		$page = false;

		if ($relativeUri === '') { // pokud vede request jen na doménu, vyžaduje se homepage v základním jazyce

			$lang = $langs[0];
			$page = $this->siteExplorer->getHomepage($lang);
			if ($page === false) {
				return null;
			}

		} else {

			// parsování jazyka
			if (!$this->defaultLangInUrl) { // pokud se v URL vyžaduje i defaultní jazyk, postupuje se jinak
				if (mb_strlen($relativeUri) == 2 and in_array($relativeUri, $langs) and $relativeUri !== $langs[0]) { // request vede na homepage ve specifickém jazyce

					$lang = $relativeUri;
					$page = $this->siteExplorer->getHomepage($lang);
					if ($page === false) {
						return null;
					}

				} else {
					if (isset($relativeUri[2]) and $relativeUri[2] !== '/') { // pokud má URL alespoň tři znaky a zároveň třetím znakem není lomítko -> jazyk = default
						$lang = $langs[0];
					} else {
						$lang = substr($relativeUri, 0, 2);
						if (in_array($lang, $langs)) {
								$relativeUri = substr($relativeUri, 3);
						} else {
							$lang = $langs[0];
						}
					}
				}
			} else {
				$length = mb_strlen($relativeUri);
				if ($length < 2) {
					return null;
				} elseif (isset($relativeUri[2]) and $relativeUri[2] !== '/') {
					return null;
				} else {

					$lang = substr($relativeUri, 0, 2);
					if (in_array($lang, $langs)) {
						$relativeUri = substr($relativeUri, 3);

						if ($relativeUri === false) {
							// opět se požaduje homepage
							$page = $this->siteExplorer->getHomepage($lang);
							if ($page === false) {
								return null;
							}
						}

					} else {  // request vede na neexistující jazyk
						return null;
					}	
				}
			}

			// získáváme stránku podle URL
			if ($page === false) {
				try {
					$page = $this->siteExplorer->matchPageByUrl($lang, $relativeUri);

				} catch (InvalidArgumentException $e) {

					// stránku, která by měla URL přesně rovné požadovanému, nemáme, a tak zjišťujeme, co by se tak od nás asi mohlo chtít...
					$relativeUriPieces = explode('/', $relativeUri);
					$piecesCount = count($relativeUriPieces);

					try {
						$pages = $this->siteExplorer->matchPagesByUrlFragment($lang, $relativeUri);
					} catch (InvalidArgumentException $e) {
						return null; // nic nevyhovuje, konečná
					}

					$stack = array();
					for ($i = 1; $i <=2; $i++) {

						$piece = array_pop($relativeUriPieces);

						$actualUri = '';

						if (empty($relativeUriPieces)) { // pokud jsme zkrátili pole na nulu, jediná možnost je, že jde o přetíženou homepage
							//$relativeUriPieces[] = '';
							foreach ($pages as $page) {
								if ($page['homepage'] == 1) {
									$actualUri = $page['url'];
								}
							}
						} else {
							$actualUri = implode('/', $relativeUriPieces);
						}
						
						if ($i == 1) {
							$stack['potentialId'] = $piece;
						}

						if (isset($pages[$actualUri])) {

							if ($i == 1) {
								$piece = $piece !== '' ? $piece : null; // předcházíme tomu, aby se nám do action a id dostaly prázdné řetězce
								if ($pages[$actualUri]['overloadAbility'] === '<action>') {
									$stack['action'] = $piece;
								} elseif ($pages[$actualUri]['overloadAbility'] === '<id>') {
									$stack['id'] = $piece;
								} elseif ($pages[$actualUri]['overloadAbility'] === '<action>/<id>') {
									$stack['action'] = $piece;
								}
							} elseif ($i == 2) {
								if ($pages[$actualUri]['overloadAbility'] === '<action>/<id>') {
									$stack['id'] = $stack['potentialId'];
									$stack['action'] = $piece !== '' ? $piece : null;
								} else {
									return null; // neznámé schéma přetěžování akce a předávání id
								}
							}

							unset($stack['potentialId']);

							$page = array_merge((array) $pages[$actualUri], $stack);
							unset($page['url']); // dále se s ní již nikde nepracuje
							break;
						}
					}
				}
			}
		}

		$params = $httpRequest->getQuery();

		if (!isset($page['pageId'])) {
			return null;
		}

		$params['lang'] = $lang;
		$params['pageId'] = (int) $page['pageId'];
		$params['action'] = $page['action'];
		$params['homepage'] = $page['homepage'];
		$params['published'] = $page['published'];

		if (isset($page['id'])) {
			$params['id'] = $page['id'];
		}

		return new PresenterRequest(
			'Front:'.$page['presenter'], // DatabaseRouter se používá pouze ve Front modulu
			$httpRequest->getMethod(),
			$params,
			$httpRequest->getPost(),
			$httpRequest->getFiles(),
			array(PresenterRequest::SECURED => $httpRequest->isSecured())
		);
	}


	/**
	 * Generuje absolutní URL z presenter requestu
	 * @param  IHttpRequest
	 * @param  PresenterRequest
	 * @return string|NULL
	 */
	public function constructUrl(PresenterRequest $appRequest, IHttpRequest $httpRequest)
	{

		$params = $appRequest->getParams();

		if (!isset($params['pageId'])) {
			return null;
		}

		try {
			$page = $this->siteExplorer->matchPageById($params['pageId']);
		} catch (InvalidArgumentException $e) {
			return null;
		}

		if ($appRequest->getPresenterName() !== 'Front:'.$page['presenter']) { // snaha o přepsání presenteru
			return null;
		}

		// Následující parametry není zapotřebí připojovat do URL, BeeRouter si je zjistí v match() části.
		unset($params['pageId'], $params['lang'], $params['lang'], $params['homepage'], $params['published']);

		$relativeUri = '';
		$actionSuffix = '';
		$idSuffix = '';

		$langs = explode(',', Environment::getConfig('siteSettings')->langs);

		if ($page['homepage'] == 1) {
			if ($page['lang'] === $langs[0] and !$this->defaultLangInUrl) {
				$relativeUri = '';
			} else {
				$relativeUri = $page['lang'];
			}
		} else {
			if ($page['lang'] === $langs[0] and !$this->defaultLangInUrl) {
				$relativeUri = $page['url'];
			} else {
				$relativeUri = $page['lang'].'/'.$page['url'];
			}
		}

		if ($page['overloadAbility'] === '<id>') {
			if (isset($params['id'])) {
				$relativeUri .= ($relativeUri !== '' ? '/' : '').$params['id'];
			}
			unset($params['id']);
		} elseif ($page['overloadAbility'] === '<action>') {
			if (isset($params['action']) and $params['action'] !== $page['action']) {
				$relativeUri .= ($relativeUri !== '' ? '/' : '').$params['action'];
			}
			unset($params['action']);
		} elseif ($page['overloadAbility'] === '<action>/<id>') {
			if (isset($params['action']) and $params['action'] !== $page['action']) {
				$relativeUri .= ($relativeUri !== '' ? '/' : '').$params['action'];
			}
			if (isset($params['id'])) {
				$relativeUri .= ($relativeUri !== '' ? '/' : '').$params['id'];
			}
			unset($params['action'], $params['id']);
		}

		if (isset($params['action']) and $params['action'] === $page['action']) {
			unset($params['action']);
		}

		$uri = $httpRequest->getUri();
		$outputUri = ($this->flags & self::SECURED ? 'https://' : 'http://').$uri->getAuthority().$uri->getScriptPath().$relativeUri;

		$sep = ini_get('arg_separator.input');
		$query = http_build_query($params, '', $sep ? $sep[0] : '&');
		if ($query != '') {
			$outputUri .= '?' . $query;
		}
		return $outputUri;

	}

}